﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.InteropServices;
using System.Text;
using System.Threading.Tasks;
using Plugin;

namespace TestPlugin
{
    /// <summary>
    /// Simple test plugin for #CSpect ZX Spectrum Emulator. The plugin logs OUTs to set the BORDER colour to the console output.
    /// Build and copy TestPlugin.dll to CSpect directory. Launch CSpect.exe from a console.
    /// TestPlugin by Marco Varesio.
    /// https://retrobits.itch.io
    /// https://retrobits.altervista.org
    /// THIS COMES WITH NO WARRANTY! USE AT YOUR OWN RISK!!!
    /// #CSpect ZX Spectrum Emulator by Mike Dailly (http://cspect.org/).
    /// </summary>
    public class TestPlugin : iPlugin
    {
        // Will work on Windows only.
        [DllImport("kernel32.dll")]
        static extern bool AttachConsole(int dwProcessId);
        private const int ATTACH_PARENT_PROCESS = -1;

        private iCSpect _CSpect;

        private ConsoleColor[] consoleColors = {
            ConsoleColor.Black, 
            ConsoleColor.Blue, 
            ConsoleColor.Red,
            ConsoleColor.Magenta,
            ConsoleColor.Green,
            ConsoleColor.Cyan,
            ConsoleColor.Yellow,
            ConsoleColor.Gray
        };

        public bool Write(eAccess type, int port, byte value)
        {
            // Maybe the condition on value is not completely correct.
            if (port == 0xFE && value <= 7)
            {
                if (value < 4)
                    Console.ForegroundColor = ConsoleColor.White;
                else
                    Console.ForegroundColor = ConsoleColor.Black;

                Console.BackgroundColor = consoleColors[value];
                Console.WriteLine($"Border: {value}");
            }
            return false;
        }

        public byte Read(eAccess type, int address, out bool isvalid)
        {
            throw new NotImplementedException();
        }

        public List<sIO> Init(iCSpect cSpect)
        {
            AttachConsole(ATTACH_PARENT_PROCESS);

            this._CSpect = cSpect;
            
            // Register for writes to 0xFE port.
            sIO io = new sIO(0xFE, eAccess.Port_Write);

            Console.WriteLine($"Test plugin initialised");

            return new[] {io}.ToList();
        }

        public void Quit()
        {
            Console.WriteLine($"Bye bye!");
        }
    }
}
